﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ParallelUtil
{
    /// <summary>
    /// Extension methods to assist in the processing of aggregate exceptions
    /// 
    /// </summary>
    public static class AggregateExceptions
    {
        /// <summary>
        /// Catch a given exception, and remove it from the set of aggregte exceptions
        /// </summary>
        /// <typeparam name="T">Type of exception to catch</typeparam>
        /// <param name="exceptions">The aggregate exception</param>
        /// <param name="catchBlock">Code to handle the exception</param>
        /// <returns>A new aggregate exception with the handled exception removed</returns>
       public static AggregateException Catch<T>(this AggregateException exceptions, Action<T> catchBlock) where T : Exception
        {
            return exceptions.Catch<T>( e => { catchBlock(e); return null; });
        }

        /// <summary>
        /// Catch a given exception, with the option to continue to 
        /// </summary>
        /// <typeparam name="T">Type of exception to catch</typeparam>
        /// <param name="exceptions">The aggregate exception</param>
        /// <param name="catchBlock">Code to handle the exception</param>
        /// <returns>original exception or new exception to be included in the aggregate exception.  Returning null removes original</returns>
        public static AggregateException Catch<T>(this AggregateException exceptions, Func<T,Exception> catchBlock ) where T: Exception
        {
            List<Exception> unhandled = new List<Exception>();

            foreach (Exception e in exceptions.InnerExceptions)
            {
                T error = e as T;
                if (error != null)
                {
                    Exception toThrow;

                    toThrow = catchBlock(error);
                    if (toThrow != null)
                    {
                        unhandled.Add(toThrow);
                    }
                    
                }
                else
                {
                    unhandled.Add(e);
                }
            }

            return new AggregateException(unhandled.ToArray());
        }

        public static void ThrowUnHandled(this AggregateException exceptions)
        {
            if (exceptions.InnerExceptions.Count > 0)
            {
                throw exceptions;
            }
        }

    }
    
}
