﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Runtime.InteropServices;

namespace ParallelUtil
{
    public static class Cores
    {
        /// <summary>
        /// Returns the total number of processing cores on this machine, could be a combination of physical and hyperthreading cores
        /// </summary>
        public static int Max
        {
            get
            {

                return Environment.ProcessorCount;
            }
        }

        /// <summary>
        /// Returns the number of Physical cores on this machine, for example an i7 has 4 physical cores and 4 hyperthreading cores,
        /// this property would return just 4 representing the 4 physical cores.
        /// </summary>
        public static int PhysicalCores
        {
            get
            {
                int nBufferElements = 20;

                SYSTEM_LOGICAL_PROCESSOR_INFO[] buffer;
                int bufferSize;
                int returnLength;

                bool success;
                do
                {
                    buffer = new SYSTEM_LOGICAL_PROCESSOR_INFO[nBufferElements];
                    bufferSize = buffer.Length * Marshal.SizeOf(typeof(SYSTEM_LOGICAL_PROCESSOR_INFO));
                    returnLength = bufferSize;

                    success = Win32.GetLogicalProcessorInformation(buffer, out returnLength);
                    nBufferElements = returnLength / Marshal.SizeOf(typeof(SYSTEM_LOGICAL_PROCESSOR_INFO));
                }
                while (returnLength > bufferSize);

                if (!success)
                {
                    throw new ParallelUtilException("Failed to Get Logical Processor Information, only supported on Vista + OS's");
                }

                int nRealCores = 0;
                int nResults = returnLength / Marshal.SizeOf(typeof(SYSTEM_LOGICAL_PROCESSOR_INFO));

                for (int nResult = 0; nResult < nResults; nResult++)
                {
                    if (buffer[nResult].Relationship == LOGICAL_PROCESSOR_RELATIONSHIP.RelationProcessorCore)
                    {
                        if (buffer[nResult].Flags > 0)
                        {
                            nRealCores++;
                        }
                    }
                }

                return nRealCores;
            }
        }

        /// <summary>
        /// Returns the number of cores assigned to this process, this is both physical and hyperthreading cores
        /// Also used to set the number of cores to assign to this process.
        /// </summary>
        public static int CoresInUse
        {
            get
            {
                IntPtr cores =
                Process.
                    GetCurrentProcess()
                    .ProcessorAffinity;

                int nCores = 0;
                while (cores != IntPtr.Zero)
                {
                    if (((int)cores & 1) == 1)
                    {
                        nCores++;
                    }
                    cores = (IntPtr)((int)cores >> 1);
                }
                return nCores;
            }

            set
            {
                if ((value < 1) || (value > Environment.ProcessorCount))
                {
                    throw new ArgumentException("Illegal number of cores");
                }

                int rightShift = 1;
                // If this is a hyperthreaded cpu
                // make sure we select  only physical cores, assuming we have sufficient
                int realCores = PhysicalCores;

                if ((realCores < Max) && (value <= realCores))
                {
                    rightShift = 2;
                }

                int cores = 1;
                for (int nShift = 0; nShift < value - 1; nShift++)
                {
                    cores = 1 | (cores << rightShift);
                }

                Process.GetCurrentProcess().ProcessorAffinity = (IntPtr)cores;
            }
        }
    }
}
