﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace ParallelUtil
{

    /// <summary>
    /// Can be used to provide non integer or stepped based Parallel loops using
    /// Parallel.ForEach instead of Parallel.For
    /// Relies  heavily on delegate invocation so for absolute performance code the loop by hand and multiply the loop index
    /// </summary>
    /// <typeparam name="T">type of the sequence value</typeparam>
    public class Sequence<T> : IEnumerable<T>
    {
        public T Start { get; private set; }
       
        private Func<T, T> Increment;
        private Predicate<T> IsEndOfSequence;

        public Sequence(T start ,  Func<T,T> increment , Predicate<T> isEndOfSequence)
        {
            if (increment == null) throw new ArgumentException("Must provide an increment method");
            if (isEndOfSequence == null) throw new ArgumentException("Must provide an end of sequence method");

            Start = start;
            
            Increment = increment;
            IsEndOfSequence = isEndOfSequence;
        }
        #region IEnumerable<T> Members

        public IEnumerator<T> GetEnumerator()
        {
            T current = Start;
            do
            {
                yield return current;
                current = Increment(current);
            }
            while(!IsEndOfSequence(current));
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return GetEnumerator();
        }

        #endregion
    }

    public class DoubleSequence : Sequence<double>
    {
        public DoubleSequence(double start , double end , double step ) : 
            base( start , c => c+step , c => c>end )
        {

        }
    }

    public class IntegerSequence : Sequence<int>
    {
        public IntegerSequence(int start, int end, int step)
            : base(start, c => c + step, c => c >end)
        {

        }
    }

}
